/*----------------------------------------------------------------------------*/
/* SPI_Flash.c                                                                */
/*----------------------------------------------------------------------------*/
/*
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Name      : HY16F3981_SPI
 * Date      : 2016/09/13
 * Version   : V0.1
 * Function  : HY16F3981 use SPI interface to communicate with SPI Flash(W25X40)
 * Remark    :
 ******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "DrvGPIO.h"
#include "DrvSPI32.h"
#include "DrvCLOCK.h"
#include "SPI_Flash.h"
#include "my define.h"

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*/
/* SPI SubRoutines                                                            */
/*----------------------------------------------------------------------------*/
//******************************************************************************
// For W25X40, Manufacturer ID: 0xEF; Device ID: 0x12
// For W25X16, Manufacturer ID: 0xEF; Device ID: 0x14
unsigned int SpiFlash_ReadMidDid(void)
{
  unsigned int au32SourceData;
  unsigned int au32DestinationData;

  DrvGPIO_ClrBit(Flash_PORT,SCS_PIN); // /CS: active

  DrvSPI32_BitLength(8);  // configure transaction length as 8 bits
  // Send Command: 0x90, Read Manufacturer/Device ID
  au32SourceData = sFLASH_CMD_REMS<<24;
  DrvSPI32_Write(au32SourceData);
  while((spi_00>>BUF) & 0x01);  // Wait Busy

  DrvSPI32_BitLength(24);  // configure transaction length as 24 bits
  DrvSPI32_Write(0x0);  // Send 24-bit '0', dummy
  while((spi_00>>BUF) & 0x01);  // Wait Busy

  DrvSPI32_BitLength(16);  // configure transaction length as 16 bits
  DrvSPI32_Write(0xFFFFFFFF);  // dump Rx register
  while((spi_00>>BUF) & 0x01);  // Wait Busy
  au32DestinationData=DrvSPI32_Read();

  DrvGPIO_SetBit(Flash_PORT,SCS_PIN); // /CS: de-active
  return  (au32DestinationData & 0xFFFF);
}

//******************************************************************************
void SpiFlash_ChipErase(void)
{
  unsigned int au32SourceData;

  DrvGPIO_ClrBit(Flash_PORT,SCS_PIN);  // /CS: active
  DrvSPI32_BitLength(8);  // configure transaction length as 8 bits
  // send Command: 0x06, Write enable
  au32SourceData = sFLASH_CMD_WREN<<24;  
  DrvSPI32_Write(au32SourceData);
  while((spi_00>>BUF) & 0x01);  // Wait Busy
  DrvGPIO_SetBit(Flash_PORT,SCS_PIN);  // /CS: de-active

  DrvGPIO_ClrBit(Flash_PORT,SCS_PIN);  // /CS: active
  au32SourceData = sFLASH_CMD_CE<<24;  // send Command: 0xC7, Chip Erase
  DrvSPI32_Write(au32SourceData);
  while((spi_00>>BUF) & 0x01);  // Wait Busy
  DrvGPIO_SetBit(Flash_PORT,SCS_PIN);  // /CS: de-active

  SpiFlash_WaitReady();
}

//******************************************************************************
void SpiFlash_SectorErase(unsigned int StartAddress)
{
  unsigned int au32SourceData;

  DrvGPIO_ClrBit(Flash_PORT,SCS_PIN);  // /CS: active
  DrvSPI32_BitLength(8);  // configure transaction length as 8 bits
  // send Command: 0x06, Write enable
  au32SourceData = sFLASH_CMD_WREN<<24;
  DrvSPI32_Write(au32SourceData);
  while((spi_00>>BUF) & 0x01);  // Wait Busy
  DrvGPIO_SetBit(Flash_PORT,SCS_PIN);  // /CS: de-active

  DrvGPIO_ClrBit(Flash_PORT,SCS_PIN);  // /CS: active
  au32SourceData = sFLASH_CMD_SE<<24;  // send Command: 0xD8, Sector Erase
  DrvSPI32_Write(au32SourceData);
  while((spi_00>>BUF) & 0x01);  // Wait Busy

  DrvSPI32_BitLength(24);  // configure transaction length as 24 bits
  au32SourceData = StartAddress<<8;
  DrvSPI32_Write(au32SourceData);
  while((spi_00>>BUF) & 0x01);  // Wait Busy
  DrvGPIO_SetBit(Flash_PORT,SCS_PIN);  // /CS: de-active

  SpiFlash_WaitReady();
}

//******************************************************************************
void SpiFlash_ReadData(unsigned char *DataBuffer, unsigned int StartAddress, unsigned int ByteCount)
{
  unsigned int au32SourceData;
  unsigned int au32DestinationData;
  unsigned int Counter;

  DrvGPIO_ClrBit(Flash_PORT,SCS_PIN);  // /CS: active

  DrvSPI32_BitLength(8);  // configure transaction length as 8 bits
  // send Command: 0x03, Read data
  au32SourceData = sFLASH_CMD_READ<<24;
  DrvSPI32_Write(au32SourceData);  //
  while((spi_00>>BUF) & 0x01);  // Wait Busy

  DrvSPI32_BitLength(24);  // configure transaction length as 24 bits
  au32SourceData = StartAddress<<8;
  DrvSPI32_Write(au32SourceData);  //
  while((spi_00>>BUF) & 0x01);  // Wait Busy

  DrvSPI32_BitLength(8);  // configure transaction length as 8 bits
  for (Counter = 0; Counter < ByteCount; Counter++)
  {
    DrvSPI32_Write(0xFFFFFFFF);  // dump Rx register
    while((spi_00>>BUF) & 0x01);  // Wait Busy

    au32DestinationData=DrvSPI32_Read();  // dump Rx register
    DataBuffer[Counter] = (unsigned char) au32DestinationData;
  }

  DrvGPIO_SetBit(Flash_PORT,SCS_PIN);  // /CS: de-active
}

//******************************************************************************
void SpiFlash_PageProgram(unsigned char *DataBuffer, unsigned int StartAddress, unsigned int ByteCount)
{
  unsigned int au32SourceData;
  unsigned int Counter;

  DrvGPIO_ClrBit(Flash_PORT,SCS_PIN);  // /CS: active
  DrvSPI32_BitLength(8);  // configure transaction length as 8 bits
  // send Command: 0x06, Write enable
  au32SourceData = sFLASH_CMD_WREN<<24;
  DrvSPI32_Write(au32SourceData);
  while((spi_00>>BUF) & 0x01);  // Wait Busy
  DrvGPIO_SetBit(Flash_PORT,SCS_PIN);  // /CS: de-active

  DrvGPIO_ClrBit(Flash_PORT,SCS_PIN);  // /CS: active
  // send Command: 0x02, Page program
  au32SourceData = sFLASH_CMD_WRITE<<24;
  DrvSPI32_Write(au32SourceData);
  while((spi_00>>BUF) & 0x01);  // Wait Busy

  DrvSPI32_BitLength(24);  // configure transaction length as 24 bits
  au32SourceData = StartAddress<<8;
  DrvSPI32_Write(au32SourceData);  //
  while((spi_00>>BUF) & 0x01);  // Wait Busy

  DrvSPI32_BitLength(8);  // configure transaction length as 8 bits
  for (Counter = 0; Counter < ByteCount; Counter++)
  {
    // send data to program
    au32SourceData = DataBuffer[Counter]<<24;
    DrvSPI32_Write(au32SourceData);
    while((spi_00>>BUF) & 0x01);  // Wait Busy
  }
  DrvGPIO_SetBit(Flash_PORT,SCS_PIN);  // /CS: de-active
  SpiFlash_WaitReady();
}

//******************************************************************************
void SpiFlash_WaitReady(void)
{
  unsigned int au32SourceData;
  unsigned int au32DestinationData;

  do
  {
    // configure transaction length as 16 bits
    DrvSPI32_BitLength(16);

    DrvGPIO_ClrBit(Flash_PORT,SCS_PIN);  // /CS: active

    au32SourceData = 0x05FF<<16;  // send Command: 0x05, Read status register 1
    DrvSPI32_Write(au32SourceData);  //
    while((spi_00>>BUF) & 0x01);  // Wait Busy
    au32DestinationData=DrvSPI32_Read();  // dump Rx register
    au32DestinationData = au32DestinationData & 1;

    DrvGPIO_SetBit(Flash_PORT,SCS_PIN);  // /CS: de-active
  }
  while (au32DestinationData != 0); // check the BUSY bit
}

/*----------------------------------------------------------------------------*/
/* SPI Initialization Subroutines                                             */
/*----------------------------------------------------------------------------*/
void InitalSPI(void)
{
  //Set SPI input pin
  DrvGPIO_Open(Flash_PORT,Flash_MOSI|Flash_SCLK|Flash_CS,E_IO_OUTPUT);  //CS=PT2.0 output
  DrvGPIO_Open(Flash_PORT,Flash_MISO,E_IO_INPUT);
  DrvGPIO_SetPortBits(Flash_PORT,Flash_CS);
  DrvSPI32_Open(E_DRVSPI_MASTER2,E_DRVSPI_TYPE0,2,3);
  // Master2,3-wire mode
  // Type0 : CPHA=0 CPOL=0
  // 2 : PT2.0=CS PT2.1=CK P2.2=MISO PT2.3=MOSI
  // 3 : CLOCK/8
  DrvSPI32_Enable();
}

